function LogDecorator(ZhuLogHelper) {
    /* Widget Getters */
    var getContentWidget = function(gridWidget) {
        return gridWidget.widgets[0].widgets[0];
    };

    /* Field Names */
    const ROUTE_FIELD_NAME              = 'ROUTE';
    const ROUTE_LABEL_FIELD_NAME        = 'ROUTE_LABEL';
    const TIME_FIELD_NAME               = 'TIMESTAMP';
    const USER_ID_FIELD_NAME            = 'USER_ID';
    const USER_NAME_FIELD_NAME          = 'USER_NAME';
    const PRODUCT_ID_FIELD_NAME         = 'PRODUCT_ID';
    const CONTENT_FIELD_NAME            = 'CONTENT';
    const HTTP_RESPONSE_CODE_FIELD_NAME = 'HTTP_RESPONSE_CODE';
    const REQUEST_TYPE_FIELD_NAME       = 'REQUEST_TYPE';
    const REQUEST_ID_FIELD_NAME         = 'REQ_ID';
    const TYPE_FIELD_NAME               = 'TYPE';
    const CONTAINER_FIELD_NAME_NAME     = 'CONTAINER_NAME';
    const WIDGET_NAME_FIELD_NAME        = 'WIDGET_NAME';
    const ORGANIZATION_ID_FIELD_NAME    = 'ORGANIZATION_ID';

    /* Field Labels */
    const USER_NAME_FIELD_LABEL    = 'User Name';
    const ROUTE_LABEL_FIELD_LABEL  = 'Action';

    /* Field Templates */
    const TEXT_EDIT_TEMPLATE       = 'field/text-edit.html';
    const SELECT_NATIVE_TEMPLATE   = 'field/select-native.html';
    const SELECT_MULTIPLE_TEMPLATE = 'field/select-multiple.html';
    const DATE_INTERVAL_TEMPLATE   = 'zh-interval-component#field/interval.html';
    const DATETIME_TEMPLATE        = 'field/datetime.html';

    var filterFieldsInAdvancedFieldGroup = [REQUEST_ID_FIELD_NAME, TYPE_FIELD_NAME, HTTP_RESPONSE_CODE_FIELD_NAME, CONTENT_FIELD_NAME];
    var fieldsNotUpdateTemplate          = [ROUTE_LABEL_FIELD_NAME];

    var updateWidgetFields = function (widget) {
        var routeField          = ZhuLogHelper.getField(widget, ROUTE_FIELD_NAME);
        var userNameField       = Util.clone(routeField);
        var routeLabelField     = Util.clone(routeField);

        userNameField.name     = USER_NAME_FIELD_NAME;
        userNameField.label    = USER_NAME_FIELD_LABEL;
        routeLabelField.name   = ROUTE_LABEL_FIELD_NAME;
        routeLabelField.label  = ROUTE_LABEL_FIELD_LABEL;

        ZhuLogHelper.moveField(widget, USER_ID_FIELD_NAME,        1);
        ZhuLogHelper.addField (widget, userNameField,             2);
        ZhuLogHelper.moveField(widget, TIME_FIELD_NAME,           3);
        ZhuLogHelper.moveField(widget, CONTAINER_FIELD_NAME_NAME, 4);
        ZhuLogHelper.moveField(widget, WIDGET_NAME_FIELD_NAME,    5);
        ZhuLogHelper.moveField(widget, ROUTE_FIELD_NAME,          6);
        ZhuLogHelper.addField (widget, routeLabelField,           7);

        ZhuLogHelper.editField(widget, ROUTE_LABEL_FIELD_NAME, {
            valueField: ROUTE_FIELD_NAME,
            descriptionField: ROUTE_LABEL_FIELD_NAME,
            template: SELECT_NATIVE_TEMPLATE,
            dataSource: {
                data: window.routesTranslate || []
            },
            filterProperties: {
                template: SELECT_MULTIPLE_TEMPLATE,
                operator: 'IN'
            },
            showOnList: true,
            maxWidth: '12%',
            class: 3
        });

        ZhuLogHelper.editField(widget, USER_ID_FIELD_NAME,        { showOnList: true, maxWidth: '10%', class: 4 });
        ZhuLogHelper.editField(widget, USER_NAME_FIELD_NAME,      { showOnList: true, maxWidth: '15%', class: 4 });
        ZhuLogHelper.editField(widget, CONTAINER_FIELD_NAME_NAME, { showOnList: true, maxWidth: '15%', class: 3 });
        ZhuLogHelper.editField(widget, WIDGET_NAME_FIELD_NAME,    { showOnList: true, maxWidth: '15%', class: 3 });
        ZhuLogHelper.editField(widget, TIME_FIELD_NAME,           { showOnList: true, maxWidth: '10%', class: 4 });
        ZhuLogHelper.editField(widget, ROUTE_FIELD_NAME,          { showOnList: true, maxWidth: '15%', class: 3 });
        ZhuLogHelper.editField(widget, TYPE_FIELD_NAME,           { showOnList: true, maxWidth: '8%' , class: 3 });

        ZhuLogHelper.removeField(widget, PRODUCT_ID_FIELD_NAME);
        ZhuLogHelper.removeField(widget, ORGANIZATION_ID_FIELD_NAME);
    };

    var updateWidgetFieldsGridOnly = function(gridWidget) {
        gridWidget.fields.forEach(function(field) {
            if(field.name && !(filterFieldsInAdvancedFieldGroup.indexOf(field.name) != -1)) {
                field.fieldGroup = null;
            }
            if(field.filterProperties) {
                if(fieldsNotUpdateTemplate.indexOf(field.name) == -1) {
                    field.template = field.filterProperties.template;
                }
                if(field.filterProperties.template == TEXT_EDIT_TEMPLATE) {
                    field.filterProperties.operator = 'LIKE';
                }
            }
        });

        //General filter fields
        ZhuLogHelper.editField(gridWidget, TIME_FIELD_NAME, {
            template: DATETIME_TEMPLATE,
            dateFormat: "DD/MM/YYYY",
            timeFormat: "HH:mm:ss",
            displayFormat: "DD/MM/YYYY HH:mm:ss",
            "datetime-dateFormat": "dd/mm/yy",
            "datetime-timeFormat": "HH:ii:ss",
            filterProperties: {
                isVisible: true,
                showOnForm: true,
                label: 'Date',
                template: DATE_INTERVAL_TEMPLATE,
                dateFormat: "DD/MM/YYYY",
                displayFormat: "DD/MM/YYYY",
                intervals: [ {
                    "label": "Última semana",
                    "period": "last-week"
                }, {
                    "label": "Último mês",
                    "period": "last-month"
                } ]
            }
        });

        //Advanced filter fields
        ZhuLogHelper.editField(gridWidget, TYPE_FIELD_NAME, { class: 4 });
        ZhuLogHelper.editField(gridWidget, HTTP_RESPONSE_CODE_FIELD_NAME, {
            class: 4,
            isVisible: false,
            filterProperties: {
                isVisible: true,
                operator : 'LIKE'
            }
        });
        ZhuLogHelper.editField(gridWidget, REQUEST_ID_FIELD_NAME, {
            class: 4,
            isVisible: false,
            filterProperties: {
                isVisible: true,
                operator : 'LIKE'
            }
        });
        ZhuLogHelper.editField(gridWidget, CONTENT_FIELD_NAME, {
            isVisible: false,
            fieldGroup: 'advancedFilter',
            template : TEXT_EDIT_TEMPLATE,
            filterProperties: {
                isVisible: true,
                operator : 'LIKE',
                class    : 12
            }
        });
    };

    var updateWidgetFieldsFormOnly = function(formWidget) {
        ZhuLogHelper.editField(formWidget, HTTP_RESPONSE_CODE_FIELD_NAME, { class: 2 });
        ZhuLogHelper.editField(formWidget, REQUEST_TYPE_FIELD_NAME,       { class: 3 });
    };

    this.decorateWidget = function(gridWidget) {
        updateWidgetFields(gridWidget);
        updateWidgetFieldsGridOnly(gridWidget);

        var formWidget = getContentWidget(gridWidget);
        updateWidgetFields(formWidget);
        updateWidgetFieldsFormOnly(formWidget);

        gridWidget.currentRowDefaultFilter = {
            TIMESTAMP: {
                start : moment().add(-1, 'days').format('DD/MM/YYYY'),
                end   : moment().format('DD/MM/YYYY'),
                period: undefined
            }
        };
    };

    this.normalizeRows    = function(rows) { return rows; };
    this.normalizeFilters = function(filters) {};
}

Configuration(function(ContextRegister){
    ContextRegister.register('LogDecorator', LogDecorator);
});
